#include "node.h"
#include "vieter_tree.h"

struct vieter_tree {
  uint64_t size;
  vieter_tree_node *root;
};

vieter_tree *vieter_tree_init() { return calloc(1, sizeof(vieter_tree)); }

uint64_t vieter_tree_size(vieter_tree *tree) { return tree->size; }

vieter_tree_error vieter_tree_insert(vieter_tree *tree, uint64_t key,
                                     void *data) {
  if (tree->size == 0) {
    tree->root = vieter_tree_node_init();
    tree->root->key = key;
    tree->root->data = data;
    tree->size = 1;

    return vieter_tree_ok;
  }

  vieter_tree_error res = vieter_tree_node_insert(tree->root, key, data);

  if (res != vieter_tree_ok) {
    return res;
  }

  tree->size++;

  return vieter_tree_ok;
}

vieter_tree_error vieter_tree_search(void **out, vieter_tree *tree,
                                     uint64_t key) {
  if (tree->size == 0) {
    return vieter_tree_not_present;
  }

  return vieter_tree_node_search(out, tree->root, key);
}

vieter_tree_error vieter_tree_remove(void **out, vieter_tree *tree,
                                     uint64_t key) {
  if (tree->size == 0) {
    return vieter_tree_not_present;
  }

  vieter_tree_error res = vieter_tree_node_remove(out, tree->root, key);

  if (res != vieter_tree_ok) {
    return res;
  }

  tree->size--;

  return vieter_tree_ok;
}
